open Format
open Highlight


let report_error ppf exn =
  let report ppf = function
    | SyntaxErrors.LexingError e -> SyntaxErrors.report_lexing_error ppf e

    | SyntaxErrors.ParsingError e -> SyntaxErrors.report_parsing_error ppf e

    | Unif.UnifError e ->
        fprintf ppf "%a" (Unif.print_unif_error ~output:Options.Ascii) e

    | TypingErrors.TypingError e -> TypingErrors.report_typing_error ppf e

    | ConstraintsSolving.InferenceError (e, meaning) ->
        ConstraintsSolvingErrors.constraint_meaning meaning e ppf

    | ConstraintsSolvingErrors.InferenceErrorReplayed (e, meaning, c) ->
        ConstraintsSolvingErrors.constraint_meaning ~c meaning e ppf

    | x -> fprintf ppf "@]"; raise x in

  fprintf ppf "@[%a@]@." report exn


let error_to_string = Utils.format_into_string report_error
