open GraphicTypes
open Nodes
open Constraints


let () = #{HashNodes}

let constrained c n =
  List.exists (fun { edge_nodes = (n1, n2) } ->
                 Nodes.equal n n1 || Nodes.equal n n2) c.constraints#get
;;

let rec interior_unconstrained ?skip c n =
  let ok n = match skip with
    | None -> false
    | Some n' -> Nodes.equal n n'
  in
  try
    dfs_interior (constraints_succs c)
      ~pre:(fun n -> if constrained c n && ok n = false then raise Exit) n;
    true
  with Exit -> false
;;


let remove_implied_existentials c =
  let remove n = dfs_interior children
    ~pre:(fun n' -> if Nodes.equal n n' = false then c.exists#remove n') n
  in
  List.iter remove (c.root :: c.exists#get);
;;


(* Remove n from the list of existentials, turning all
   nodes at the frontier into existentials. Should be called
   only when n is unconstrained *)
let rec remove_existential c n =
  dfs_interior (constraints_succs c)
    ~frontier:(fun n -> Utils.debug "%d" (nid n); c.exists#add_unique n)
    ~pre:c.exists#remove
   n;
;;

(* Remove all existential nodes that are unconstrained *)
let remove_all_unconstrained c =
  let rec aux = function
    | [] -> ()
    | n :: q ->
        if interior_unconstrained c n then remove_existential c n;
        aux q;
  in aux c.exists#get;
;;

(* Is an existential node bound on a forall *)
let forall_existentially_constrained c f =
  List.exists (fun n -> match !!n.bound with
                 | Root -> Nodes.equal f c.root
                 | Bound { binder = n' } -> Nodes.equal f n')
    c.exists#get
;;


let all_nodes c =
  let l = ref [] in
  (dfs_constraints c).GraphicTypesAux.generic_dfs
    ~pre:(fun n -> l := n :: !l) c.root;
  !l
