open GraphicTypes
open Nodes
open Constraints
open GraphicSyntacticTranslations
open Format
open Lexing
open Locations

let print_position pp pos =
  fprintf pp "%S %d %d %d" pos.pos_fname pos.pos_lnum pos.pos_bol pos.pos_cnum;
;;


let cmp_loc_inner_first loc1 loc2 =
  match Pervasives.compare loc1.loc_end.pos_cnum loc2.loc_end.pos_cnum with
    | 0 -> Pervasives.compare loc2.loc_start.pos_cnum loc1.loc_start.pos_cnum
    | x -> x
;;


let cmp_ti_inner_first ti1 ti2 =
  cmp_loc_inner_first (snd ti1) (snd ti2)
;;


let print_expr_type sugar_env ppf t loc =
  print_position ppf loc.loc_start;
  fprintf ppf " ";
  print_position ppf loc.loc_end;
  fprintf ppf "@.type(@.  ";
  fprintf ppf "@[%a@]"
    (SyntacticTypesPrinting.print_polytype_no_alpha ~sugar_env) t;
  fprintf ppf "@.)@.";
;;

let dump_ppf = ref None

let print_expr_roots sugar_env l =
  match !dump_ppf with
    | None -> ()
    | Some ppf ->
        let l' = List.sort cmp_ti_inner_first l in
        let tr = graph_to_syntactic_aux () in
        let aux (n, loc) =
          if loc.loc_ghost = false then
            let t =
              if is_forall !!n.constr then (
(*                reset_forall n; *)
                tr.node_type (child1 n);
              ) else
                tr.node_type n
            in
            print_expr_type sugar_env ppf t loc
        in
        List.iter aux l'
