open SyntacticTypes
open TypeConstructors
open Locations
open Highlight
open Format


type typing_error =
  | UnboundVar of AST.value * location
  | UnknownTypeConstructor of typeconstr_name * location
  | UnknownInductiveConstructor of constr_name * location
  | IncorrectArity of typeconstr_name * int * int * location
  | NonDistinctParameters of typevar * location
  | NonDistinctConstructors of constr_name * location
  | UnboundTypeVar of typevar * location
  | MLFCoercionMLConstraint of UnknownconstrTypes.expanded_polytype * location
  | IncorrectSugarAnnotation of typeconstr_name * int * location

exception TypingError of typing_error

exception MLFTypeMLFExpansion

let typing_error e = raise (TypingError e)

let report_typing_error ppf = function
  | UnboundVar (v, loc) ->
      print_loc ppf loc;
      fprintf ppf "Unbound var '%s'." (AST.value_s v)

  | UnknownTypeConstructor (c, loc) ->
      print_loc ppf loc;
      fprintf ppf "Unbound type constructor '%s'." (typeconstr_s c)

  | UnknownInductiveConstructor (c, loc) ->
      print_loc ppf loc;
      fprintf ppf "Unbound constructor '%s'." (constr_s c)

  | IncorrectArity (c, ac, au, loc) ->
      print_loc ppf loc;
      fprintf ppf "The type constructor %s expects %d argument(s) but is used with %d argument(s)." (typeconstr_s c) ac au

  | NonDistinctParameters (v, loc) ->
      print_loc ppf loc;
      fprintf ppf
        "The type variable '%s appears multiple times in this parameters declaration."
        (typevar_s v)

  | UnboundTypeVar (v, loc) ->
      print_loc ppf loc;
      fprintf ppf
        "The type variable '%s is unbound in this type declaration."
        (typevar_s v)

  | NonDistinctConstructors (c, loc) ->
      print_loc ppf loc;
      fprintf ppf
        "The constructor '%s' is defined more than once in this type declaration." (constr_s c)

  | MLFCoercionMLConstraint (_t, loc) ->
      print_loc ppf loc;
      fprintf ppf
        "Only ML type annotations (without universal quantification) are permitted in ML constraints"

  | IncorrectSugarAnnotation (c, arity, loc) ->
      print_loc ppf loc;
      fprintf ppf
        "This sugar annotation is incorrect, the type constructor %s has arity %d" (typeconstr_s c) arity
